@ECHO OFF
SETLOCAL ENABLEDELAYEDEXPANSION

CALL:GETBINEQ
IF "%BINEQ%"=="" GOTO END
ECHO Testing executable "%BINEQ%".

SET MKLINK-AVAILABLE=FALSE
SET WARN-NOADMIN=FALSE
MKLINK >NUL 2>&1
IF NOT ERRORLEVEL 9009 (
	NET SESSION >NUL 2>&1
	IF ERRORLEVEL 1 (
		SET WARN-NOADMIN=TRUE
	) ELSE (
		SET MKLINK-AVAILABLE=TRUE
	)
)

CALL:GETTEMPNAME
MD "%TMPDIR%"
ECHO Hello World!>"%TMPDIR%\Hello"
COPY "%TMPDIR%\Hello" "%TMPDIR%\Hello2">NUL
ECHO Bye World>"%TMPDIR%\Bye"
ECHO Such a Size!>"%TMPDIR%\Same Size"
CALL:CREATE-BIG-FILE "%TMPDIR%\Big1"
CALL:CREATE-BIG-FILE "%TMPDIR%\Big2"
MD "%TMPDIR%\Dir"
IF %MKLINK-AVAILABLE%==TRUE (
	MKLINK/J "%TMPDIR%\DirLinkHard" "%TMPDIR%\Dir">NUL
	MKLINK/D "%TMPDIR%\DirLinkSoft" "%TMPDIR%\Dir">NUL
	MKLINK/H "%TMPDIR%\HelloLinkHard" "%TMPDIR%\Hello">NUL
	MKLINK "%TMPDIR%\HelloLinkSoft" "%TMPDIR%\Hello">NUL
	MKLINK "%TMPDIR%\Hello2LinkSoft" "%TMPDIR%\Hello2">NUL
)
SET TEST_COUNT=0
SET ALL_OK=TRUE

"%BINEQ%"/Q "%TMPDIR%\Hello" "%TMPDIR%\Hello"
CALL:ASSERT-EQ "A file matches itself"
"%BINEQ%"/Q "%TMPDIR%\Hello" "%TMPDIR%\Bye"
CALL:ASSERT-DIFF "Files with different sizes don't match"
"%BINEQ%"/Q "%TMPDIR%\Hello" "%TMPDIR%\Same Size"
CALL:ASSERT-DIFF "Files with different contents don't match"
"%BINEQ%"/Q "%TMPDIR%\Big1" "%TMPDIR%\Big2"
CALL:ASSERT-EQ "Large file comparison works"
"%BINEQ%"/Q "%TMPDIR%\Dir" "%TMPDIR%\Dir"2>NUL
CALL:ASSERT-ERR "Folders cannot be opened"
"%BINEQ%" "%TMPDIR%\Hello" /Q 2>NUL
CALL:ASSERT-ERR "An error occurs if only one argument is specified"
"%BINEQ%"/Q "%SYSTEMROOT%\SYSTEM32\SHELL32.DLL" "%TMPDIR%\Big1" 2>NUL
CALL:ASSERT-EQ "File system redirection is not enabled"
IF %MKLINK-AVAILABLE%==TRUE (
	"%BINEQ%"/Q "%TMPDIR%\DirLinkHard" "%TMPDIR%\DirLinkHard"2>NUL
	CALL:ASSERT-ERR "Hard links to folders cannot be opened"
	"%BINEQ%"/Q/R "%TMPDIR%\DirLinkSoft" "%TMPDIR%\DirLinkSoft"2>NUL
	CALL:ASSERT-ERR "Soft links to folders cannot be opened"
	"%BINEQ%"/Q/R "%TMPDIR%\HelloLinkSoft" "%TMPDIR%\Hello2LinkSoft"
	CALL:ASSERT-DIFF ^
	"Soft links to different files do not match (larger reparse data)"
	"%BINEQ%"/Q "%TMPDIR%\HelloLinkHard" "%TMPDIR%\HelloLinkSoft" "%TMPDIR%\Hello2"
	CALL:ASSERT-EQ ^
	"Matching a hard link, a soft link and a file with the same contents"
	"%BINEQ%"/Q "%TMPDIR%\HelloLinkHard"/R "%TMPDIR%\HelloLinkSoft"
	CALL:ASSERT-DIFF "A hard link with a soft link don't match with option /R"
)

DEL/S/Q "%TMPDIR%">NUL
IF %ALL_OK%==TRUE (
	ECHO All tests are OK.
) ELSE (
	ECHO Some tests are NOT OK.
)
IF %WARN-NOADMIN%==TRUE (
	ECHO Rerun this script with administrator privileges to perform more tests.
)
:END
PAUSE
GOTO:EOF

:GETBINEQ
SET/A COUNT=0
SET BINEQ=
FOR /F "DELIMS=" %%A IN ('DIR/S/B "%~DP0\BINEQ.EXE" 2^>NUL') DO (
	SET/A COUNT+=1
	SET BINEQ.!COUNT!=%%A
	IF !COUNT!==9 GOTO LAB1
)
IF %COUNT%==0 (
	ECHO BINEQ.EXE not found!
) ELSE IF %COUNT%==1 (
	SET BINEQ=%BINEQ.1%
) ELSE (
:LAB1
	ECHO BINEQ.EXE has been found in different locations:
	SET CHOICES=
	FOR /L %%I IN (1,1,%COUNT%) DO (
		SET CHOICES=!CHOICES!%%I
		ECHO. %%I. !BINEQ.%%I!
	)
	CHOICE/? >NUL 2>&1
	IF NOT ERRORLEVEL 9009 (
		CHOICE/C !CHOICES! /N /M "Please, enter the number of the executable you want to test:"
		FOR /L %%I IN (%COUNT%,-1,1) DO (
			IF ERRORLEVEL %%I (
				SET BINEQ=!BINEQ.%%I!
				ECHO.
				GOTO:EOF
			)
		)
	) ELSE (
		SET/P CHOICE=Please, enter the number of the executable you want to test: 
		FOR /L %%I IN (%COUNT%,-1,1) DO (
			IF !CHOICE!==%%I (
				SET BINEQ=!BINEQ.%%I!
				ECHO.
				GOTO:EOF
			)
		)
	)
	ECHO No executable specified.
)
GOTO:EOF

:GETTEMPNAME
SET TMPDIR=%TMP%\BINEQ-TEST-%RANDOM%-%TIME:~6,5%.tmp
IF EXIST "%TMPDIR%" GOTO:GETTEMPNAME
GOTO:EOF

:CREATE-BIG-FILE
COPY "%SYSTEMROOT%\SYSTEM32\SHELL32.DLL" "%1">NUL
GOTO:EOF

:ASSERT
SET/A TEST_COUNT+=1
SET/A NEXT_ERRORLEVEL=%1+1
SET STATUS=NOT OK
IF ERRORLEVEL %1 IF NOT ERRORLEVEL %NEXT_ERRORLEVEL% SET STATUS=OK
IF NOT "%STATUS%"=="OK" SET ALL_OK=FALSE
ECHO. %TEST_COUNT%. %STATUS%: %~2
GOTO:EOF

:ASSERT-EQ
CALL:ASSERT 0 %1
GOTO:EOF

:ASSERT-DIFF
CALL:ASSERT 1 %1
GOTO:EOF

:ASSERT-ERR
CALL:ASSERT 2 %1
GOTO:EOF
